from torch import nn
from frozen_bert import FrozenBert


class BertBiLSTM(FrozenBert):
    def __init__(self, bert=None, inherit_config=False, name_override=None):
        self.name = "BertBiLSTM"
        if name_override is not None:
            if isinstance(name_override, str):
                self.name = name_override

        super().__init__(bert, inherit_config)

        self.bilstm = nn.LSTM(self.bert.config.hidden_size, self.bert.config.hidden_size, bidirectional=True)
        self.dense = nn.Linear(2 * self.bert.config.hidden_size, self.bert.config.vocab_size)
        self.log_softmax = nn.LogSoftmax(dim=-1)

    def forward(self, input_ids, attention_mask, token_type_ids=None):
        frozen_bert_output = super().forward(input_ids, attention_mask, token_type_ids=token_type_ids)

        lstm_output, _ = self.bilstm(frozen_bert_output[0])
        dense_output = self.dense(lstm_output)

        log_softmax_output = self.log_softmax(dense_output)
        return log_softmax_output
