from torch import nn
from transformers import BertModel
from frozen_bert import FrozenBert


class BertDoubleDense(FrozenBert):
    def __init__(self, bert=None, inherit_config=False):
        # Inherit layers from parent
        super(FrozenBert, self).__init__(bert, inherit_config)

        self.name = "BertSingleDense"

        # Add additional layer(s)
        self.dense1 = nn.Linear(self.bert.config.hidden_size, self.bert.config.hidden_size)  # 1st Dense layer
        self.relu = nn.ReLU()  # ReLU activation function
        self.dense2 = nn.Linear(self.bert.config.hidden_size, self.bert.config.vocab_size)  # 2nd Dense layer
        self.log_softmax = nn.LogSoftmax(dim=-1)

    def forward(self, input_ids, attention_mask, token_type_ids=None):
        frozen_bert_output = super().forward(input_ids, attention_mask, token_type_ids=token_type_ids)

        dense_output1 = self.dense1(frozen_bert_output[0])
        relu_output1 = self.relu(dense_output1)
        dense_output2 = self.dense2(relu_output1)
        relu_output2 = self.relu(dense_output2)

        log_softmax_output = self.log_softmax(relu_output2)
        return log_softmax_output


if __name__ == "__main__":
    print("You are running a model class file directly. Go run main.py or equivalent trainer instead!")
