from torch import nn
from transformers import BertModel
from frozen_bert import FrozenBert


class BertSingleDense(FrozenBert):
    def __init__(self, bert=None, inherit_config=False, name_override=None):
        self.name = "BertSingleDense"
        if name_override is not None:
            if isinstance(name_override, str):
                self.name = name_override

        # Inherit layers from parent
        super().__init__(bert, inherit_config)

        # Add additional layer(s)
        self.dense = nn.Linear(self.bert.config.hidden_size, self.bert.config.vocab_size)  # 1x Dense layer
        self.relu = nn.ReLU()  # ReLU activation function
        self.log_softmax = nn.LogSoftmax(dim=-1)

    def forward(self, input_ids, attention_mask, token_type_ids=None):
        # Obtain context embeddings from frozen BERT model
        frozen_bert_output = super().forward(input_ids, attention_mask, token_type_ids=token_type_ids)

        # Pass context embeddings through unfrozen dense layer
        dense_output = self.dense(frozen_bert_output[0])  # Last hidden layer of BERT - [1] is pooled

        # Obtain results of ReLU activation function applied to dense output
        relu_output = self.relu(dense_output)
        log_softmax_output = self.log_softmax(relu_output)
        return log_softmax_output


if __name__ == "__main__":
    print("You are running a model class file directly. Go run main.py or equivalent trainer instead!")