from sklearn.model_selection import KFold
from pytorch_lightning import Trainer, LightningModule
from torch.utils.data import DataLoader, Subset
from tqdm import tqdm


# Used if you ever want to verify if some percentages add to 100 - useless
def verify_split_percentages(test_split_percentage, train_split_percentage, validate_split_percentage=None,
                             verbose=False):
    if validate_split_percentage is None:
        corrected_validate_split_percentage = 0
    elif validate_split_percentage < 0:
        corrected_validate_split_percentage = 0
    else:
        corrected_validate_split_percentage = validate_split_percentage

    total_percentage = test_split_percentage + validate_split_percentage + train_split_percentage

    if total_percentage == 100:
        return True
    else:
        if verbose:
            output = f"Invalid Test Train Validation Splits Provided:\n"
            output += f"\ntest_split_percentage: {test_split_percentage}%"
            output += f"\nvalidate_split_percentage: {corrected_validate_split_percentage}% - original: " \
                      f"{validate_split_percentage}"
            output += f"\ntrain_split_percentage: {train_split_percentage}%\n"
            output += f"\nTotal percentage: {total_percentage}%."
            print(output)
        return False


def cross_validate(model, test_dataloader, k_folds=10):
    # kfold doesnt work on loaders
    dataset = list(test_dataloader.dataset)

    # create k-folder
    kfolder = KFold(n_splits=k_folds, shuffle=True)

    progress_bar = tqdm(enumerate(kfolder.split(dataset)), total=k_folds, desc="Folds")

    results = []
    for fold, (train_ids, val_ids) in enumerate(kfolder.split(dataset)):
        # Subsets made according to ids returned by the kfolder
        train_subsample = Subset(dataset, train_ids)
        val_subsample = Subset(dataset, val_ids)

        # Load corresponding dataset
        # train_data_loader = DataLoader(train_subsample, batch_size=test_dataloader.batch_size, shuffle=True)
        val_data_loader = DataLoader(val_subsample, batch_size=test_dataloader.batch_size, shuffle=False)

        trainer = Trainer(max_epochs=0)
        trainer.validate(model=model, dataloaders=val_data_loader)

    trainer = Trainer(max_epochs=0)
    trainer.test(dataloaders=test_dataloader)
