from torch import nn
from transformers import BertModel


# Wrapper Class for BERT - Freezes BERT Layers but otherwise does nothing but wrap bert
class FrozenBert(nn.Module):
    def __init__(self, bert=None, inherit_config=False):
        super(FrozenBert, self).__init__()

        # Define BERT layer
        if bert is None:
            self.bert = BertModel.from_pretrained('bert-base-uncased')
        else:
            self.bert = bert

        # Set config
        if inherit_config:
            self.config = self.bert.config

        # Freeze Bert Parameters
        for param in self.bert.parameters():
            param.requires_grad = False

    def forward(self, input_ids, attention_mask, token_type_ids=None):
        # Return Context Embeddings
        return self.bert(input_ids=input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)


if __name__ == "__main__":
    print("You are running a model class file directly. Go run main.py or equivalent trainer instead!")
