import datetime
import warnings
import re

def get_datetime_string():
    return datetime.strftime('%Y%m%d_%H%M%S')


def get_class_member_string(class_, verbose=True):
    members = [attr for attr in dir(class_) if not callable(getattr(class_, attr)) and not attr.startswith("__")]
    if verbose:
        print(members)
    return members


def build_model_name(model, model_name=None, add_time_to_model_name=False, model_name_appendix=""):
    if model_name is None:
        try:
            model_name = model.name
        except ValueError:
            model_name = {type(model).__name__}
            warnings.warn(f"Model Name Property Not Found for {model_name}. If this is not an acceptable name, please "
                          f"revise your model to have model.name as a property.")

    if add_time_to_model_name:
        time_string = get_datetime_string()
        model_name_appendix += "_" + time_string

    model_name += model_name_appendix

    return model_name


def fix_requirements(text=None, file_path=None, print_file=False):
    if file_path is None and text is None:
        raise ValueError("Nothing supplied to remove")
    if file_path is not None:
        print("Fixing ", file_path)
        with open(file_path, 'r') as file:
            lines = file.readlines()
        filtered_lines = [line for line in lines if not re.search(r'file://', line)]
        output_file_path = "requirements.txt"
        with open(output_file_path, 'w') as file:
            file.writelines(filtered_lines)
        if print_file:
            with open(output_file_path, 'r') as file:
                print(file.read())

    else:
        lines = text.split('\n')
        filtered_lines = [line for line in lines if not re.search(r'file://', line)]
        filtered_text = '\n'.join(filtered_lines)
        print(filtered_text)


def create_pip_installs_from_requirements(text=None, file_path=None, print_file=False, add_magic=True):
    if file_path is None and text is None:
        raise ValueError("Nothing supplied to remove")
    if file_path is not None:
        print("Fixing ", file_path)
        with open(file_path, 'r') as file:
            lines = file.readlines()
        if add_magic:
            prepend = "!"
        else:
            prepend = ""
        filtered_lines = [prepend+"pip install "+line.split("==")[0] for line in lines if not re.search(r'file://',
                                                                                                             line)]
        output_file_path = "requirements_pip_installs.txt"
        with open(output_file_path, 'w') as file:
            file.writelines(filtered_lines)
        if print_file:
            with open(output_file_path, 'r') as file:
                print(file.read())

    else:
        lines = text.split('\n')
        filtered_lines = [line for line in lines if not re.search(r'file://', line)]
        filtered_text = '\n'.join(filtered_lines)
        print(filtered_text)


if __name__ == '__main__':
    file_path = 'requirements.txt'
    fix_requirements(file_path=file_path)
    create_pip_installs_from_requirements(file_path=file_path)

